#include "MC68EZ328.h"
#include "vivo.h"

unsigned char display[4800]
    __attribute__((aligned(0x20000))); /* may not cross 128k boundaries */

char hexnibble[] = "0123456789abcdefg";

void tx(unsigned char c)
{
  while (UTX & UTX_BUSY)
    ;
  UTX_TXDATA = c;
}

void txhex(unsigned char c)
{
  tx(hexnibble[(c>>4)&0xf]);
  tx(hexnibble[c&0xf]);
}

void puts(char *s)
{
  while(*s)
    tx(*s++);
}

void init()
{
  LSSA = (int)display;

  LXMAX = LCD_XDIM;
  LVPW = LCD_XDIM/16;
  LYMAX = LCD_YDIM;
  LPICF = LPICF_GS_BW | LPICF_PBSIZ_4;
  LCKCON = LCKCON_LCDON;
  PCSEL = 0;
  LCD_SOMETHING_PDIR |= LCD_SOMETHING;
  LCD_SOMETHING_PDATA |= LCD_SOMETHING;

  MAX686_DAC_PDIR |= MAX686_DAC_UP|MAX686_DAC_DOWN;

  {
    int i;
    for (i=0; i<127; i++) {
      MAX686_DAC_PDATA |= MAX686_DAC_DOWN;
      MAX686_DAC_PDATA &= ~MAX686_DAC_DOWN;
    }
    for (i=0; i<30; i++) {
      MAX686_DAC_PDATA |= MAX686_DAC_UP;
      MAX686_DAC_PDATA &= ~MAX686_DAC_UP;
    }
  }

  MAX686_NLCDON_PDIR |= MAX686_NLCDON;
  MAX686_NLCDON_PDATA |= MAX686_NLCDON;

}

void putpixel(int x, int y, unsigned char color)
{

  if (x > LCD_XDIM || x < 0)
    return;
  if (y > LCD_YDIM || y < 0)
    return;
  if (color)
    display[(x + y*LCD_XDIM)>>3] |= 0x80 >> (x & 0x7);
  else
    display[(x + y*LCD_XDIM)>>3] &= ~(0x80 >> (x & 0x7));

}

void
touch_init()
{
  BBADS7843_NCS_PDATA |= BBADS7843_NCS;
  BBADS7843_NCS_PDIR |= BBADS7843_NCS;

  PESEL &= ~7;
  SPIMCONT |= 7 << SPIMCONT_DATA_RATE_SHIFT;
  SPIMCONT |= SPIMCONT_ENABLE;
  SPIMCONT |= 7; /* bit count-1 */
}

int
touch_sample(char axis)
{
  int i;
  int result;

  BBADS7843_NCS_PDATA &= ~BBADS7843_NCS;
  SPIMDATA = 0b10000011 | (axis ? BBADS7843_SWITCHES_Y : BBADS7843_SWITCHES_X);
  SPIMCONT |= SPIMCONT_XCH;
  while(SPIMCONT & SPIMCONT_XCH)
    ;
  SPIMDATA = 0;
  SPIMCONT |= SPIMCONT_XCH;
  while(SPIMCONT & SPIMCONT_XCH)
    ;
  result = SPIMDATA & 0xff;
  SPIMDATA = 0;
  SPIMCONT |= SPIMCONT_XCH;
  while(SPIMCONT & SPIMCONT_XCH)
    ;
  result <<= 8;
  result |= SPIMDATA & 0xff;
  BBADS7843_NCS_PDATA |= BBADS7843_NCS;
  return result >>4;
}

int main(int argc, char *arv[])
{
  int i;
  init();

  {
    unsigned char v = 0x80;
    while (v) {
      for(i=0; i<sizeof(display); i++)
	display[i]=v;
      v >>= 1;
    }
  }
  cls:
    for(i=0; i<sizeof(display); i++)
      display[i]=0;


  touch_init();

  while (BUTTON_PDATA & BUTTON) {
    int x,y;
    x = touch_sample(0);
    y = touch_sample(1);
    x += touch_sample(0);
    y += touch_sample(1);
    x += touch_sample(0);
    y += touch_sample(1);
    x += touch_sample(0);
    y += touch_sample(1);
    y>>=2;
    x>>=2;

    x-= TOUCH_XMIN;
    y-= TOUCH_YMIN;

#define XRANGE (TOUCH_XMAX - TOUCH_XMIN)
#define YRANGE (TOUCH_YMAX - TOUCH_YMIN)

    x-= XRANGE /2;
    y-= YRANGE /2;

    x = x * LCD_XDIM /XRANGE;
    y = y * LCD_YDIM /YRANGE;

    putpixel(LCD_XDIM/2 + x,
	     LCD_YDIM/2 + y,
	     1);
  }
  goto cls;
  return 0;
}

